/*

 Fading.h -- Extension to the VaRGB library.
 Copyright (C) 2015 Martin Jeske.


 Created on: 2015-05-20

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 3 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 See file LICENSE.txt for further informations on licensing terms.

*/
#include "includes/VaRGBConfig.h"
#include <math.h>

#ifdef VaRGB_ENABLE_CURVE_FADING

#include "includes/Curves/Fading.h"


//#define FLOATIFY_DIVISIONS
namespace vargb {
namespace Curve {




Fading::Fading(VaRGBColorValue target_red, VaRGBColorValue target_green,
		VaRGBColorValue target_blue, VaRGBTimeValue trans_time_seconds, float gamma_correction):
		Curve(target_red, target_green, target_blue, trans_time_seconds),
		gamma(gamma_correction)	
		{


}

#ifdef VaRGB_CLASS_DESTRUCTORS_ENABLE
Fading::~Fading() {
	// TODO Auto-generated destructor stub
}
#endif

void Fading::setTick(VaRGBTimeValue setTo, IlluminationSettings* initial_settings)
{
	
	
	resetCurrentSettings(initial_settings);	
    
     
     
     
	if (setTo < 1)
	{
		return;
	}

	tick_count = setTo;
	
	settings_req_update = true;

}

void Fading::tick(uint8_t num)
{


	for (uint8_t i=0; i<num; i++)
	{
		tick_count++;

		if (tick_count >= curve_target.transition_ticks)
		{
			curve_completed = true;
		}
         
		for (uint8_t c=0; c< VaRGB_NUM_COLORS; c++)
		{
 /*
-------------------------------------------      
R[i] = floor((z - 1) * (i / (n - 1)) ^ (1/?) + 0.5)
R[i] = floor((VaRGB_GENERIC_PLAT_COLORVALUE_MAX - 1) * (tick_count / (curve_target.transition_ticks - 1)) ^ (1/?) + 0.5)

mit

n = Anzahl der Eintrge in der Look-Up-Tabelle (z.B. 256)

i = Laufindex [0..n-1]

z = Anzahl der linearen Stufen, die die PWM-Routine wiedergeben kann (z.B. 65536) 
	*/
		  if ( curve_target.values[c] == 0 ) current_settings.values[c] = 0;
		  else{
		  //VaRGBColorValue  nextValue = floor(pow(2, log_b * (tick_count+1) / curve_target.values[c]));
		  VaRGBColorValue nextValue =   curve_target.transition_ticks *1.0 * pow(  (tick_count *1.0/ (curve_target.transition_ticks - 1)*1.0), (1/gamma)) + 0.5;
		  if(nextValue > curve_target.values[c]) nextValue = curve_target.values[c];
		  current_settings.values[c] = floor( nextValue);
		  }
		}
    settings_req_update = true;
	}
}


} /* namespace Curve */
} /* namespace vargb */

// VaRGB_ENABLE_CURVE_Fading
#endif

