#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <iostream>
#include "VaRGB.h"
#include "VaRGBCurves.h"


//#include "includes/VaRGBPlatform.h"
//#include "includes/Schedule.h"
//#include "includes/Curve.h"

#define byte uint8_t 
#define RED_PIN 18
#define GREEN_PIN 23
#define BLUE_PIN  24
double  PWM_SCALER = 1/double(VaRGB_GENERIC_PLAT_COLORVALUE_MAX);
FILE *fp;
void  INThandler(int sig)
{
    
     signal(sig, SIG_IGN);
    //turn all pins off
    fprintf(fp, "%i=%f\n", RED_PIN,0.0);
    fflush(fp);
    fprintf(fp, "%i=%f\n", GREEN_PIN,0.0);
    fflush(fp);
    fprintf(fp, "%i=%f\n", BLUE_PIN,0.0);
    fflush(fp);
    
    fclose(fp);
    exit(0);
}

/* *** The VaRGB driver object *** */

// callback declarations.
// We need a color-setting callback and a another 
// callback for when a schedule has run its course, 
// in order to create our VaRGB driver.

// these are forward declarations, the actual functions
// are defined below.  You can change the names, but not 
// the signatures (void return and their respective parameters):
void setColorValues(vargb::ColorSettings * set_to);
void scheduleCompleteCB(vargb::Schedule * sched);


// with our callbacks recognizable by the compiler as functions of the 
// correct types, we create the VaRGB driver object:
vargb::VaRGB RGB_prog(setColorValues, scheduleCompleteCB);
#define number_of_schedules 4
vargb::Schedule * available_schedules[number_of_schedules] = {
      new vargb::Schedule(),
      new vargb::Schedule(),
      new vargb::Schedule(),
      new vargb::Schedule(),
  
};
/* *** Curves -- the RGB transition specifications *** */

// As mentioned, a schedule is comprised of one or more curves.  These curves 
// describe the behaviour of the RGB settings with time.
//
// There are a few kinds of curves available with the library, like:
//
//  * Linear -- each RGB component varies linearly with time (a constant 
//              slope), like a simple "fade" in or out
//  
//  * Sine -- RGB components vary as sine waves, so quickly around their 
//              mid-point and slower at the extremes
//
//  * Flasher -- RGB components are alternated between 0 and their setting
//
// In addition, there are "combo" curves which allow you to combine 
// other curves with AND or OR logic.  These are left for another example.

// now, let's create a few of these curves...

// a single "white" flasher, for schedule 0

#define num_flashes      20
#define time_flashing    6
vargb::Curve::Flasher * flasher_curve = new vargb::Curve::Flasher(VaRGB_COLOR_MAXVALUE, 
                                                        VaRGB_COLOR_MAXVALUE, VaRGB_COLOR_MAXVALUE, 
                                                        time_flashing, num_flashes);

// a green-tinted sine wave, for schedule 0: say 5 cycles of 2 seconds each 
// (total run time of 10 seconds)
#define sine_runtime                 6
#define sinewave_seconds_per_cycle   2
vargb::Curve::Sine * green_sine_wave = new vargb::Curve::Sine(500, VaRGB_COLOR_MAXVALUE, 500, 
                                                          sine_runtime, sinewave_seconds_per_cycle);



// a sequence of linear "fades" for schedule 2, store in an array for convenience
#define num_curves_schedule1  5
vargb::Curve::Linear * schedule1_curves[num_curves_schedule1] = {
 
                            // start black
                            new vargb::Curve::Linear(0,0,0,       0), 
                            // go to ~1/2 red, over one 5 seconds
                            new vargb::Curve::Linear(500,0,0,     5), 
                            // go to red+blue, over 2s
                            new vargb::Curve::Linear(1000,0,1000, 2), 
                            // back down to ~1/2 red over 2s
                            new vargb::Curve::Linear(500,0,0,     2), 
                            // fade to black for 5s
                            new vargb::Curve::Linear(0,0,0,       5), 
};

vargb::Curve::Fading * schedule1_curves_fading[num_curves_schedule1] = {
 
                            // start black
                            new vargb::Curve::Fading(0,0,0,       0,0.22), 
                            // go to ~1/2 red, over one 5 seconds
                            new vargb::Curve::Fading(500,0,0,     5, 0.22), 
                            // go to red+blue, over 2s
                            new vargb::Curve::Fading(1000,0,1000, 2, 0.22), 
                            // back down to ~1/2 red over 2s
                            new vargb::Curve::Fading(500,0,0,     2, 0.22), 
                            // fade to black for 5s
                            new vargb::Curve::Fading(0,0,0,       5, 0.22), 
};

// now, let's create a few of these curves...
#define max_color_val	1000
#define half_color_val	500
// a single simple "white" flasher
#define num_flashes_log      15
#define time_flashing_log    3
vargb::Curve::Flasher first_flasher(max_color_val,
          max_color_val, max_color_val,
          time_flashing_log, num_flashes_log);



vargb::Curve::Flasher second_flasher(max_color_val,
          max_color_val, max_color_val,
          time_flashing_log, num_flashes_log);
// The Not is created with the curve to apply it to, along with a boolean to denote whether
// each of the RED, GREEN, and BLUE channels are affected.  Here we only NOT the GREEN:
vargb::Curve::Not not_first_flasher(&second_flasher, false, true, false);





// a sine wave that goes from 0 to max on all channels
#define sine_runtime_log             		8
#define sinewave_seconds_per_cycle   	        2
vargb::Curve::Sine sine_wave(max_color_val, max_color_val, max_color_val,
                             sine_runtime_log, sinewave_seconds_per_cycle);

// Now we half-rectify that sine wave by creating a Threshold curve
vargb::Curve::Threshold half_rectified_sine(&sine_wave, 511, vargb::Curve::ThresholdAbove, 512);  




// More rectification, we'll fully rectify the red and blue components of the signal by doing this:

// Create a second sine wave of the same duration and frequency as the first

vargb::Curve::Sine sine_wave_2(max_color_val, max_color_val, max_color_val,
                      sine_runtime_log, sinewave_seconds_per_cycle);

// Invert the red and blue channels, this will shift the phase of the sine wave on these channels
vargb::Curve::Not inverted_sine(&sine_wave_2, true, false, true);

// create a mask so we'll let through only the red and blue channels
vargb::Curve::Constant redandblue_mask_curve(max_color_val, 0, max_color_val, sine_runtime_log);

// mask the inverted sine, by AND-ing with the mask
vargb::Curve::AndLogic redandblue_inverted_sine(&redandblue_mask_curve, &inverted_sine);

// half-rectify our red+blue sine signals, using a Threshold
vargb::Curve::Threshold half_rectified_redandbluesine(&redandblue_inverted_sine, 511, vargb::Curve::ThresholdAbove, 512);  

// combine the half-rectified red+blue with the original half-rectified sine wave
vargb::Curve::OrLogic fully_rectified_red_and_blue(&half_rectified_redandbluesine, &half_rectified_sine);




/* grand finale */

// For our grand finale, we'll have three distinct waves for each color channel
#define grand_finale_seconds  22

// green will have a linear sawtooth, a signal that rises steadily to max, then abruptly
// drops to 0:

// create a steady rising linear value on green
vargb::Curve::Linear steady_rising(0, max_color_val, 0, grand_finale_seconds);

// create a mask for some of the lower bits, i.e. 124 == b001111100
vargb::Curve::Constant green_mask_curve(0, 124, 0, grand_finale_seconds);

// apply the mask to the linear green curve
vargb::Curve::AndLogic masked_steady_rising(&steady_rising, &green_mask_curve);

// left-shift the resulting values up, so the signal strength is very visible: 
// yay, we have a sawtooth
vargb::Curve::Shift sawtooth(&masked_steady_rising, 3, vargb::Curve::ShiftLeft);


// for the red channel, we'll have a pseudo-sawtooth, based on a sine signal
// so it will vary the rate at which it changes and the direction of the sawtooth

// create a slowly changing sine wave
vargb::Curve::Sine slow_sine(max_color_val, 0, 0, grand_finale_seconds, 10);

// create a mask for the red channel, 62 == b0000111110, so we keep the lower bits
vargb::Curve::Constant red_mask_curve(62, 0, 0, grand_finale_seconds);

// mask that slow sine wave by AND-ing with our constant value curve
vargb::Curve::AndLogic masked_sine(&slow_sine, &red_mask_curve);

// shift the signal up (left) so we get some visible action
vargb::Curve::Shift alt_sawtooth(&masked_sine, 4, vargb::Curve::ShiftLeft);


// now we have two kinds of sawtooth on the red and green channels
// combine them into one curve, by OR-ing:
vargb::Curve::OrLogic both_sawtooths(&alt_sawtooth, &sawtooth);


// create a quicker sine wave on the blue channel
vargb::Curve::Sine quick_sine(0, 0, max_color_val, grand_finale_seconds, 2);

// combine that quick sine wave with the sawtooths from before
vargb::Curve::OrLogic sawtooths_and_sine(&both_sawtooths, &quick_sine);


// one second break...
vargb::Curve::Constant rest_curve(0,0,0, 1);

// straight to black
vargb::Curve::Linear instant_black(0,0,0,0);

 
/* *** Curves -- the RGB transition specifications *** */

#define sine_runtime_phased                 40
#define sinewave_seconds_per_cycle_phased   2


// a standard sine wave, on the red channel
vargb::Curve::Sine red_sine_wave(VaRGB_COLOR_MAXVALUE, 0, 0,
                                   sine_runtime_phased, sinewave_seconds_per_cycle_phased);


// same sine wave params as the red, but on the green channel AND we set an 
// extra parameter to shift the phase of the sine wave by 90 degrees
vargb::Curve::Sine green_sine_wave_phased(0, VaRGB_COLOR_MAXVALUE, 0, 
                                   sine_runtime_phased, sinewave_seconds_per_cycle_phased, 90);



// final sinewave for blue channel, shifted by 180 degrees (basically inverted)
vargb::Curve::Sine blue_sine_wave(0, 0, VaRGB_COLOR_MAXVALUE,
                                   sine_runtime_phased, sinewave_seconds_per_cycle_phased, 180);




// now, we'd like to send all these at the same time, so we combine them all into
// a single logical curve, by doing
//
// ((RED_SINE OR GREEN_SINE) OR BLUE_SINE)


// combine the red and green:
vargb::Curve::OrLogic red_and_green(&red_sine_wave, &green_sine_wave_phased);

// combine those two with the blue:
vargb::Curve::OrLogic all_sines(&red_and_green, &blue_sine_wave);



void scheduleCompleteCB(vargb::Schedule * sched)
{
  // when a schedule has completely run through all of its transitions,
  // this callback is called, with the just-terminated schedule as 
  // the parameter.
  
  // use this function to do whatever you like, e.g. change the schedule
  // as we're doing here.
  
  // one important thing is to call resetTicks() if you just want to start
  // from the begining (of this or another schedule):
    
  RGB_prog.resetTicks();
  
  
  // in this case, we'll move on to the next schedule
  byte next_id = sched->id() + 1;
  //if (next_id >= number_of_schedules )
  if (next_id >= number_of_schedules -1)
  {
    next_id = 0; // past end, so loop back to first schedule...
  }
 // next_id =2;
  RGB_prog.setSchedule(available_schedules[next_id]);
  
}
  


void setColorValues(vargb::ColorSettings * set_to)
{
  
  // the color-setting callback, specified in this examples as this
  // "setColorCB()" function, is called whenever there a change
  // to the RGB color settings is required.  
  fprintf(fp, "%i=%f\n", RED_PIN, double(set_to->red * PWM_SCALER) );
  fflush(fp);
  
  fprintf(fp, "%i=%f\n", GREEN_PIN,double(set_to->green * PWM_SCALER) );
  fflush(fp);
  
  fprintf(fp, "%i=%f\n", BLUE_PIN,double(set_to->blue * PWM_SCALER) );
  fflush(fp);



}


int main(void)
{
    signal(SIGINT, INThandler);
    //need for initializing wiringPI delay method
    wiringPiSetupSys();
 
   
    // open the pi-blaster device file. If it fails, confirm that pi-blaster has been started
    fp = fopen("/dev/pi-blaster", "w");
    if (fp == NULL) {
        printf("Error opening file\n");
        exit(0);
     }
   
 
 /*** Callbacks *** */

// these are the callbacks used by the VaRGB driver, that we promised we would
// be defining above (the forward declarations)
  // Tell the schedules what curves each is responsible for:
  
  
 
  // add the flasher to the first schedule (0):
  available_schedules[0]->addTransition(flasher_curve);
  // then add the sinewave here too
  available_schedules[0]->addTransition(green_sine_wave);
  
  
  
  // add all the linear to the second program
  for (byte i=0; i< num_curves_schedule1; i++)
  {
    available_schedules[1]->addTransition(schedule1_curves[i]);
  }
  
  for (byte i=0; i< num_curves_schedule1; i++)
  {
    available_schedules[1]->addTransition(schedule1_curves_fading[i]);
  }
  
  
   
    // Tell the schedule what curves it is responsible for:
  // straight flasher:
  available_schedules[2]->addTransition(&first_flasher);



  // inverted green flasher:
  available_schedules[2]->addTransition(&not_first_flasher);


  // straight sine wave
  available_schedules[2]->addTransition(&sine_wave);



  // half-rectified sine wave
  // 
  available_schedules[2]->addTransition(&half_rectified_sine);



  // inverted red and blue sine
 available_schedules[2]->addTransition(&inverted_sine);

  // fully rectified
  available_schedules[2]->addTransition(&fully_rectified_red_and_blue);


  // blacken
  available_schedules[2]->addTransition(&instant_black);


  // wow me, amadeus
  available_schedules[2]->addTransition(&sawtooths_and_sine);

  // break time
  available_schedules[2]->addTransition(&rest_curve);

  available_schedules[3]->addTransition(&all_sines);

  
  // Finally, tell our VaRGB driver where to begin (schedule 0)
  RGB_prog.setSchedule(available_schedules[0]);
  while(true){
      
      RGB_prog.tickAndDelay();
  
}
    fclose(fp);
    return 0;
}



