/*

 Fading.h -- Extension to the VaRGB library.
 Copyright (C) 2015 Martin Jeske.


 Created on: 2015-05-20

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 3 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 See file LICENSE.txt for further informations on licensing terms.


 *****************************  OVERVIEW  *****************************
Fading will cycle the R-G-B components following a formula which is modelled 
on the characteristic curve of the human eye.



*/

#ifndef FADING_H_
#define FADING_H_

#include "../Curve.h"

namespace vargb {
namespace Curve {


class Fading : public Curve {
public:
	/*
	 * Fading curve constructor.
	 * Pass the target (maximum) values for red, green and blue components,
	 * along with the total curve run time (in seconds).
	 * base sets the value for the calculation. It should be between 10 - 100
	 * 
	 */
	Fading(VaRGBColorValue target_red, VaRGBColorValue target_green,
			VaRGBColorValue target_blue, VaRGBTimeValue time_seconds, float gamma_correction);

#ifdef VaRGB_CLASS_DESTRUCTORS_ENABLE
	virtual ~Fading();
#endif



	virtual void setTick(VaRGBTimeValue setTo,  IlluminationSettings* initial_settings=NULL);
	virtual void tick(uint8_t num=1);
private:
	 float gamma;	  


};

} /* namespace Curve */
} /* namespace vargb */
#endif /* FADING_H_ */
